# -*- coding: utf-8 -*-
import datetime

from odoo import api, fields, models, _


class TechTag(models.Model):
    _name = 'project.tech.tag'

    name = fields.Char('Tech. Type', size=50, tracking=True)

    project_task_ids = fields.Many2many('project.task', 'task_tech_tag_rel', 'project_task_id', 'tech_tag_id', string='Tech. Type')


class ProjectTask(models.Model):
    _inherit = 'project.task'

    start_date = fields.Date('開始日期')
    finish_date = fields.Date('結束日期')
    tech_tag_ids = fields.Many2many('project.tech.tag', 'task_tech_tag_rel', 'tech_tag_id', 'project_task_id', string='Tech. Type')

    resource_request = fields.Many2one('res.users', string='resource request', tracking=True)

    manufacture_data = fields.Many2many('mrp.production')

    @api.onchange('stage_id')
    def _task_stage_check(self):
        self.project_id._task_stage_check(self, self.stage_id)

    def cron_deadline_check(self):
        today = datetime.date.today()
        tasks = self.env['project.task'].search([])
        projects = self.env['project.project'].search([])
        for task in tasks:
            if task.date_deadline:
                if task.kanban_state != 'done':
                    if today >= task.date_deadline:
                        task.update({
                            'kanban_state': 'blocked',
                        })
                    else:
                        task.update({
                            'kanban_state': 'normal',
                        })

        for project in projects:
            for task in project.task_ids:
                if task.kanban_state == 'blocked':
                    project.update({
                        'last_update_status': 'off_track',
                    })
                    break
                else:
                    project.update({
                        'last_update_status': 'on_track',
                    })

    def cron_deadline_notification_check(self):
        today = datetime.date.today()
        projects = self.env['project.project'].search([])
        owner = []
        lowest_due_day = 100
        for project in projects:
            message = ''
            for task in project.task_ids:
                url = f"/web#id={task.id}&view_type=form&model=project.task"
                if task.date_deadline:
                    if task.kanban_state != 'done':
                        due_days = (task.date_deadline - today).days
                        if due_days == 7 or 0 < due_days <= 3:
                            message += f"任務 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{task.name}</a> 剩餘 {due_days} 日到期 </br>"
                            lowest_due_day = due_days if lowest_due_day < due_days else lowest_due_day
                            for user in task.user_ids:
                                if user not in owner:
                                    owner.append(user)
                        elif due_days == 0:
                            message += f"任務 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{task.name}</a> 將於今日到期 </br>"
                            lowest_due_day = due_days if lowest_due_day < due_days else lowest_due_day
                            for user in task.user_ids:
                                if user not in owner:
                                    owner.append(user)
                        elif due_days < 0:
                            message += f"任務 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{task.name}</a> 已過期 {abs(due_days)} 日 </br>"
                            lowest_due_day = due_days if lowest_due_day < due_days else lowest_due_day
                            for user in task.user_ids:
                                if user not in owner:
                                    owner.append(user)
            if message:
                self.action_send_notification(project, owner, lowest_due_day, message)

    def action_send_notification(self, project, owner, due_days, message):
        mrp_managers = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('kci-mrp.kci_mrp_manager_group').id)])
        email_to = ''
        for partner in owner:
            email_to += f'{partner.email}, '
        email_cc = ''
        for partner in mrp_managers.channel_partner_ids:
            email_cc += f'{partner.email}, '
        if due_days <= 0:
            managers = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('kci-project.kci_manager_group').id)])
            for partner in managers.channel_partner_ids:
                email_cc += f'{partner.email}, '

        email_values = {
            'email_to': email_to,
            'email_cc': email_cc,
            'email_from': "erp_system@kci-global.com.tw",
            'body_html': message,
            'subject': f'[KCI ERP System]專案 {project.name} 通知'
        }
        mail_template = self.env.ref('kci-project.deadline_notification_email_template')
        mail_template.send_mail(self.id, force_send=True, email_values=email_values)

    def write(self, vals):
        if self.kanban_state == 'done':
            vals['kanban_state'] = 'done'
        res = super(ProjectTask, self).write(vals)
        return res


class Project(models.Model):
    _inherit = 'project.project'

    # purchase_data = fields.One2many('purchase.order', 'purchase_project', 'Purchase Orders', copy=True, tracking=True)
    # sales_data = fields.One2many('sale.order', 'sales_project', 'Sales Orders', copy=True, tracking=True)
    # bom_data = fields.One2many('mrp.bom', 'bom_project', 'Bom', copy=True, tracking=True)

    shipping_day = fields.Date('交貨日期', tracking=True)
    accepting_day = fields.Date('驗收日期', tracking=True)

    purchase_data = fields.Many2many('purchase.order')
    sales_data = fields.Many2many('sale.order')
    bom_data = fields.Many2many('mrp.bom')

    def _task_stage_check(self, changed_task, new_task_stage):
        origin_task = self.env['project.task'].browse(str(changed_task.id)[6:])
        for type_id in self.type_ids:
            tasks = list(self.env['project.task'].search([('project_id', '=', self.id), ('stage_id', '=', type_id.id)]))
            for task in tasks:
                if str(origin_task.id) == str(task.id):
                    tasks.remove(task)
            if type_id == new_task_stage:
                tasks += origin_task
            if tasks:
                for project in self:
                    project.update({
                        'stage_id': self.env['project.project.stage'].search([('name', '=', type_id.name), ])[0].id,
                    })
                break

    def cron_project_deadline_notification_check(self):
        today = datetime.date.today()
        projects = self.env['project.project'].search([])
        message = ''
        for project in projects:
            url = f"/web#id={project.id}&view_type=form&model=project.project"
            if project.shipping_day:
                shipping_due_days = (project.shipping_day - today).days
                if shipping_due_days == 14:
                    message += f"專案 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{project.name}</a> 距離出貨日剩餘 {shipping_due_days} 日 </br>"
                elif shipping_due_days == 0:
                    message += f"專案 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{project.name}</a> 出貨日今日到期 </br>"
                elif shipping_due_days < 0:
                    message += f"專案 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{project.name}</a> 已超過出貨日 {abs(shipping_due_days)} 日 </br>"
            if project.accepting_day:
                accepting_due_days = (project.accepting_day - today).days
                if accepting_due_days == 14:
                    message += f"專案 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{project.name}</a> 距離驗收日剩餘 {accepting_due_days} 日 </br>"
                elif accepting_due_days == 0:
                    message += f"專案 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{project.name}</a> 驗收日今日到期 </br>"
                elif accepting_due_days < 0:
                    message += f"專案 <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{project.name}</a> 已超過驗收日 {abs(accepting_due_days)} 日 </br>"
        if message:
            self.action_send_notification(project, message)

    def action_send_notification(self, project, message):
        mrp_managers = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('kci-mrp.kci_mrp_manager_group').id)])
        managers = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('kci-project.kci_manager_group').id)])
        email_to = f'{project.user_id.email}'
        for partner in mrp_managers.channel_partner_ids:
            email_to += f'{partner.email}, '
        for partner in managers.channel_partner_ids:
            email_to += f'{partner.email}, '

        email_values = {
            'email_to': email_to,
            'email_from': "erp_system@kci-global.com.tw",
            'body_html': message,
            'subject': f'[KCI ERP System]專案到期日通知'
        }
        mail_template = self.env.ref('kci-project.deadline_notification_email_template')
        mail_template.send_mail(self.id, force_send=True, email_values=email_values)


# class PurchaseOrder(models.Model):
#     _inherit = 'purchase.order'
#
#     purchase_project = fields.Many2one('project.project', 'Purchase Project', index=True, required=False, default=False)
#
#
# class Sales(models.Model):
#     _inherit = 'sale.order'
#
#     sales_project = fields.Many2one('project.project', 'Sales Project', index=True, required=False, default=False)
#
#
# class Bom(models.Model):
#     _inherit = 'mrp.bom'
#
#     bom_project = fields.Many2one('project.project', 'Bom Project', index=True, required=False, default=False)
