from odoo import api, fields, models


class InternalPurchaseOrder(models.Model):
    _name = 'purchase.internal.order'
    _description = 'The internal purchase order for purchase workflow'
    _inherit = ['purchase.order', 'mail.thread', 'mail.activity.mixin']

    READONLY_STATES = {
        'purchase': [('readonly', True)],
        'done': [('readonly', True)],
        'cancel': [('readonly', True)],
    }

    partner_id = fields.Many2one('res.partner', string='Vendor', required=False, states=READONLY_STATES, change_default=True, tracking=True, domain="['|', ('company_id', '=', False), ('company_id', '=', company_id)]", help="You can find a vendor by its Name, TIN, Email or Internal Reference.")
    order_code = fields.Char('案件編號', size=100, tracking=True, default='New')
    inquiry_id = fields.Many2one('sales.inquiry', string='詢價單')
    bom_id = fields.Many2one('mrp.bom', string='bom')
    order_line = fields.One2many('purchase.order.line', 'internal_order_id', string='項目', copy=True)
    assigned_engineer = fields.Many2one('res.users', string='指定工程師')
    project = fields.Many2one('project.project', default=False)

    def action_bom_request(self):
        '''
        This function create a bom-1 for estimate a case.
        '''
        product = self.env['product.product'].create({'name': self.inquiry_id['name'], 'product_type': 'project'})
        product.product_tmpl_id.update({
            'product_type': 'project',
        })
        if self.inquiry_id['type'] == 'project':
            bom = self.env['mrp.bom'].create({
                'product_tmpl_id': product.product_tmpl_id.id,
                'status': 'internal',
                'customer_id': self.inquiry_id['customer'].id,
                'inquiry_id': self.inquiry_id.id,
                'purchase_id': self.id,
                'user_id': self.assigned_engineer.id,
                'project_id': self.project.id,
            })
            # and mention engineer
        else:
            bom = self.env['mrp.bom'].create({
                'product_tmpl_id': product.product_tmpl_id.id,
                'status': 'quotation',
                'customer_id': self.inquiry_id['customer'].id,
                'inquiry_id': self.inquiry_id.id,
                'purchase_id': self.id
            })
            for line in self.order_line:
                self.env['mrp.bom.line'].create({
                    'product_id': line.product_id.id,
                    'product_qty': line.product_qty,
                    'product_uom_id': line.product_uom.id,
                    'price_unit': line.price_subtotal,
                    'bom_id': bom.id
                })

        self.bom_id = bom.id
        self.inquiry_id['bom_data'] = bom.id

        self.action_send_notification(self.bom_id, self.inquiry_id['type'])

    def action_send_notification(self, bom_id, case_type):
        notification_ids = [((0, 0, {
            'res_partner_id': self.env.user.partner_id.id,
            'notification_type': 'inbox'}))]
        user_id = self.env.user.id
        email_to = ''
        url = f"/web#id={bom_id.id}&view_type=form&model=mrp.bom"
        message = f"已建立 BOM <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{bom_id.product_tmpl_id.name}</a> "
        group = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('kci-mrp.kci_mrp_group').id)]) if case_type == 'project' else self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('sales.kci_sales_manager_group').id)])
        group.message_post(author_id=user_id,
                           body=message + "請查收",
                           message_type='notification',
                           subtype_xmlid="mail.mt_comment",
                           notification_ids=notification_ids,
                           partner_ids=[self.env.user.id],
                           notify_by_email=False,
                           )
        self.env['res.partner'].browse(user_id).message_post(author_id=user_id,
                                                             body=message + "已寄送",
                                                             message_type='notification',
                                                             subtype_xmlid="mail.mt_comment",
                                                             notification_ids=notification_ids,
                                                             partner_ids=[self.env.user.id],
                                                             notify_by_email=False,
                                                             )
        if self.assigned_engineer:
            email_to += f'{self.assigned_engineer.email}, '
            self.assigned_engineer.partner_id.message_post(author_id=user_id,
                                                                 body=message + "已寄送",
                                                                 message_type='notification',
                                                                 subtype_xmlid="mail.mt_comment",
                                                                 notification_ids=notification_ids,
                                                                 partner_ids=[self.env.user.id],
                                                                 notify_by_email=False,
                                                                 )

        for partner in group.channel_partner_ids:
            email_to += f'{partner.email}, '
        email_values = {
            'email_to': email_to,
            'email_cc': self.env.user.partner_id.email,
            'email_from': "erp_system@kci-global.com.tw",
            'body_html': message,
        }
        mail_template = self.env.ref('kci-purchase.internal_purchase_email_template')
        mail_template.send_mail(self.id, force_send=True, email_values=email_values)


class InternalPurchaseOrderLine(models.Model):
    _name = 'purchase.internal.order.line'
    _inherit = 'purchase.order.line'

    # order_id = fields.Many2one('purchase.order', string='Order Reference', index=True, required=False, ondelete='cascade')
    internal_order_id = fields.Many2one('purchase.internal.order', string='Internal Order Reference', index=True, ondelete='cascade')


class PurchaseOrderLine(models.Model):
    _inherit = 'purchase.order.line'

    order_id = fields.Many2one('purchase.order', string='Order Reference', index=True, required=False, ondelete='cascade')
    internal_order_id = fields.Many2one('purchase.internal.order', string='Internal Order Reference', index=True, ondelete='cascade')


class PurchaseOrder(models.Model):
    _inherit = 'purchase.order'

    order_code = fields.Char('案件編號', size=100, tracking=True, default='New')
    inquiry_id = fields.Many2one('sales.inquiry', string='詢價單')
    bom_id = fields.Many2one('mrp.bom', string='bom')
