# -*- coding: utf-8 -*-
from datetime import datetime

from odoo import models, fields, api, _


class Inquiry(models.Model):
    _name = 'sales.inquiry'
    _description = 'Inquiry for sales.'
    _inherit = ['mail.thread', 'mail.activity.mixin']

    type = fields.Selection([
        ('none', '未指定'),
        ('machining-O', '加工件(MO)'),
        ('machining-D', '加工件(MD)'),
        ('buying', '市購件(B)'),
        ('operation', '維修件(O)'),
        ('project', '專案件(P)')
    ], default='none', help='Case type for case inquiry.', tracking=True, string='案件類型')
    code = fields.Char('案件編號', size=100, tracking=True, default='New')
    name = fields.Char('案件名稱', size=100, tracking=True)
    date = fields.Datetime('建立日期', default=fields.Datetime.now, readonly=True)
    purchase_data = fields.Many2one('purchase.order', tracking=True, string='採購', default=False)
    internal_purchase = fields.Many2one('purchase.internal.order', string='內部詢價單', default=False)
    bom_data = fields.Many2one('mrp.bom', tracking=True, string='BOM', default=False)
    project = fields.Many2one('project.project', default=False)
    sales_manager = fields.Many2one('hr.employee', tracking=True, string='業務主管')
    sales_person = fields.Many2one('hr.employee', default=lambda self: self.env['hr.employee'].search([('user_id', '=', self.env.user.id)]).id, tracking=True, string='業務人員')
    customer_com = fields.Many2one('res.partner', string='公司')
    customer = fields.Many2one('res.partner', tracking=True, string='客戶', domain="[('parent_id', '=', customer_com)]", required=True)
    factory = fields.Char(string='廠別', size=100)
    department = fields.Char(string='部門', size=100)
    description = fields.Char(string='說明/備註', size=100)

    photo_ids = fields.One2many('sales.inquiry.photo', 'inquiry_id', string='照片')
    survey_date = fields.Datetime(string='廠勘時間')
    detector = fields.Many2many('hr.employee', string='廠勘人員')
    survey_line_ids = fields.One2many('sales.survey.line', 'survey_id', string='項目與內容')
    survey_file = fields.Binary(string='廠勘表', tracking=True)
    survey_file_name = fields.Char('Survey Name')
    detail_file = fields.Binary(string='細項', tracking=True)
    detail_file_name = fields.Char('Detail Name')

    company_id = fields.Many2one(
        'res.company', 'Company', index=True,
        default=lambda self: self.env.company)

    @api.onchange('type')
    def _refresh_code(self):
        type_mapping = {
            'none': 'N',
            'machining-O': 'MO',
            'machining-D': 'MD',
            'buying': 'B',
            'operation': 'O',
            'project': 'P'
        }
        date = datetime.now()
        serial_number = len(self.env['sales.inquiry'].search([('code', 'like', f"{type_mapping[self.type]}-{str(date.year)[2:]}{date.month if date.month > 9 else '0' + str(date.month)}-{self.sales_person.sales_id if self.sales_person.sales_id else '00'}")]))
        serial = f'00{serial_number + 1}' if serial_number < 9 else f'0{serial_number + 1}'
        self.code = f"{type_mapping[self.type]}-{str(date.year)[2:]}{date.month if date.month > 9 else '0' + str(date.month)}-{self.sales_person.sales_id if self.sales_person.sales_id else '00'}-{serial}"

    @api.onchange('customer')
    def _refresh_customer_data(self):
        self.update({
            'factory': self.customer.factory,
            'department': self.customer.department,
            'customer_com': self.customer.parent_id,
        })

    @api.model
    def create(self, vals):
        result = super(Inquiry, self).create(vals)
        internal_order = self.env['purchase.internal.order'].create({'name': result.name, 'order_code': result.code, 'inquiry_id': result.id})
        result.internal_purchase = internal_order
        if result.type == 'project':
            project = self.env['project.project'].create({'name': result.name, 'partner_id': result.customer.id})
            result.project = project
            internal_order.update({
                'project': project.id
            })
        self.action_send_notification(internal_order)
        return result

    def action_send_notification(self, internal_order):
        notification_ids = [((0, 0, {
            'res_partner_id': self.env.user.partner_id.id,
            'notification_type': 'inbox'}))]
        user_id = self.env.user.id
        url = f"/web#id={internal_order.id}&view_type=form&model=purchase.internal.order"
        message = f"已建立內部詢價單 {internal_order.order_code} <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{internal_order.name}</a> "
        group = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('kci-purchase.kci_purchase_group').id)])
        group.message_post(author_id=user_id,
                           body=message + "請查收",
                           message_type='notification',
                           subtype_xmlid="mail.mt_comment",
                           notification_ids=notification_ids,
                           partner_ids=[self.env.user.id],
                           email_from="rex.wu@kci-global.com.tw",
                           subject="[KCI ERP System]內部詢價單通知",
                           notify_by_email=True,
                           )
        self.env["res.partner"].browse(self.env.user.partner_id.id).message_post(author_id=user_id,
                                                                                 body=message + "已寄送",
                                                                                 message_type='notification',
                                                                                 subtype_xmlid="mail.mt_comment",
                                                                                 notification_ids=notification_ids,
                                                                                 partner_ids=[self.env.user.id],
                                                                                 email_from="rex.wu@kci-global.com.tw",
                                                                                 subject="[KCI ERP System]內部詢價單通知",
                                                                                 notify_by_email=True,
                                                                                 )
        email_to = ''
        for partner in group.channel_partner_ids:
            email_to += f'{partner.email}, '
        email_values = {
            'email_to': email_to,
            'email_cc': self.env.user.partner_id.email,
            'email_from': "erp_system@kci-global.com.tw",
            'body_html': message,
        }
        mail_template = self.env.ref('sales.inquiry_email_template')
        mail_template.send_mail(self.id, force_send=True, email_values=email_values)

        # self.env['mail.message'].create({
        #     'email_from': self.env.user.partner_id.email,  # add the sender email
        #     'author_id': self.env.user.partner_id.id,  # add the creator id
        #     'model': 'mail.channel',  # model should be mail.channel
        #     'type': 'comment',
        #     'subtype_id': self.env.ref('mail.mt_comment').id, #Leave this as it is
        #     'body': "Body of the message",  # here add the message body
        #     'channel_ids': [self.env.ref('kci-purchase.kci_purchase_group').id],  # This is the channel where you want to send the message and all the users of this channel will receive message
        #     'res_id': self.env.ref('kci-sales.kci_sales_group').id,  # here add the channel you created.
        # })


class InquiryPhoto(models.Model):
    _name = 'sales.inquiry.photo'
    _description = 'photo for inquiry'
    _inherit = ['mail.thread', 'mail.activity.mixin']

    photo_file = fields.Binary('Photo', tracking=True)
    photo_file_name = fields.Char('Photo Name')

    inquiry_id = fields.Many2one('sales.inquiry', 'photo_id', index=True, ondelete='cascade')

    company_id = fields.Many2one(
        'res.company', 'Company', index=True,
        default=lambda self: self.env.company)


class SurveyLine(models.Model):
    _name = 'sales.survey.line'
    _description = 'Survey lines'
    _inherit = ['mail.thread', 'mail.activity.mixin']

    survey_items = [
        ('none', '未指定'),
        ('a0', '基本資料-確認機台數量/型號/使用製程(thin file/dry etch/wet/photo/diffusion/CMP...)'),
        ('a1', '基本資料-製程環境(真空/大氣/溫度/solvent system/跑藥液/一般環境...其他)'),
        ('a2', '基本資料-確認專案目的(提昇良率/改善報廢/cost down/multiple sourcing/oe others)'),
        ('a3', '基本資料-改善需求(提昇良率/改善報廢/cost down/multiple sourcing/oe others)'),
        ('a4', '基本資料-專案類型(sensor/視覺/改機/PLC 人機更換/光學/收值/替代品更換)'),
        ('a5', '基本資料-監控方式(PLC/人機/IPC 遠端/IPC FDC/不須上傳 FDC)'),
        ('a6', '基本資料-警報需求(蜂鳴器/三色燈)'),
        ('a7', '基本資料-照片需求(請客戶協助拍攝 or 錄影檔)'),
        ('a8', '基本資料-異常發生處理方式(現場警報人機 or 蜂鳴/停機/上傳 FDC)'),
        ('b0', 'Sensor 需求內容-Sensor 種類(LED/Laser/flowmeter/shift/vibration/MEMS/ultrasonic/acoustic/電磁場/轉速/溫度/濕度...)'),
        ('b1', 'Sensor 需求內容-品牌型號紀錄'),
        ('b2', 'Sensor 需求內容-量測精度(um/mm/%/˚C/mL/L/gal/加速度...)'),
        ('b3', 'Sensor 需求內容-錶頭數值紀錄/最大流量每分鐘'),
        ('b4', 'Sensor 需求內容-管徑需求(夾管/截管)'),
        ('b5', 'Sensor 需求內容-管材質(PTFE/鐵/塑...)'),
        ('b6', 'Sensor 需求內容-被測物材質(水/藥液/氣體...)'),
        ('b7', 'Sensor 需求內容-需求數量'),
        ('c0', '視覺辨識需求內容-辨識內容(wafer/人員/產品)'),
        ('c1', '視覺辨識需求內容-被測物尺寸大小'),
        ('c2', '視覺辨識需求內容-辨識精度(尺寸 mm/象限/圓周/顏色/形貌)'),
        ('c3', '視覺辨識需求內容-環境是否有光(黃光/白光/無/其他)'),
        ('c4', '視覺辨識需求內容-環境是否能打光(黃光/白光/or others)'),
        ('c5', '視覺辨識需求內容-安裝支架量測'),
        ('c6', '視覺辨識需求內容-相機廠牌規格數量'),
        ('c7', '視覺辨識需求內容-IPC 需求規格數量'),
        ('d0', '機構-類型：sensor 支架/相機支架/其他'),
        ('d1', '機構-需求部件(flange/o-ring/電磁防護板/or others...)'),
        ('d2', '機構-量測尺寸'),
        ('d3', '機構-需求材質(e.g. Sus/viton/PTFE...)'),
        ('e0', 'PLC 更換-原 PLC 品牌型號/照片提供(廠勘事前提供)'),
        ('e1', 'PLC 更換-是否可以提供電路圖'),
        ('e2', 'PLC 更換-安排現場讀值')
    ]

    item = fields.Selection(survey_items, string='項目', default='none', tracking=True)
    content = fields.Char(string='內容', size=100)
    remark = fields.Char(string='備註', size=100)
    survey_id = fields.Many2one('sales.inquiry', 'survey_line', index=True, ondelete='cascade', default=False)

    company_id = fields.Many2one(
        'res.company', 'Company', index=True,
        default=lambda self: self.env.company)
