# -*- coding: utf-8 -*-
import json

from odoo import models, fields, api, _
from odoo.exceptions import UserError


class SaleOrder(models.Model):
    _inherit = 'sale.order'

    @api.model
    def _get_default_note(self):
        result = '''
        <p>1. 此報價單有效期限30天止。</p>
        <p>2. 不在本報價範圍之項目，另外計價。</p>
        <p>3. 交期 </p>
        '''
        return result

    order_code = fields.Char('案件編號', size=100, tracking=True, default='New')
    version = fields.Char(size=100, string='版本', default='1')
    inquiry_id = fields.Many2one('sales.inquiry', string='詢價單')

    price_type = fields.Selection([
        ('major_item', "Major"),
        ('detail_item', "Detail")], default='detail_item', help="Item type for product.")

    parent_order = fields.Many2one('sale.order', string='母單')
    bank = fields.Many2one('sales.order.bank', string='付款帳號')
    note = fields.Html('Terms and conditions', default=_get_default_note)

    sale_order_code = fields.Char(string='訂單編號', size=100, trcking=True)
    attached_ids = fields.One2many('sales.order.attached', 'order_id', string='訂單附件')

    @api.onchange('inquiry_id')
    def _inquiry_data(self):
        self.name = self.inquiry_id.name
        self.order_code = self.inquiry_id.code
        self.partner_id = self.inquiry_id.customer

    @api.depends('order_line.price_total', 'price_type')
    def _amount_all(self):
        """
        Compute the total amounts of the SO.
        """
        for order in self:
            amount_untaxed = amount_tax = 0.0
            for line in order.order_line:
                if line.item_type == order.price_type:
                    amount_untaxed += line.price_subtotal
                    amount_tax += line.price_tax
            order.update({
                'amount_untaxed': amount_untaxed,
                'amount_tax': amount_tax,
                'amount_total': amount_untaxed + amount_tax,
            })

    name = fields.Char(string='Order Reference', required=True, copy=False, readonly=False,
                       states={'draft': [('readonly', False)]}, index=True, default=lambda self: _('New'))

    @api.depends('order_line.tax_id', 'order_line.price_unit', 'amount_total', 'amount_untaxed')
    def _compute_tax_totals_json(self):
        def compute_taxes(order_line):
            if order_line.item_type == self.price_type:
                price = order_line.price_unit * (1 - (order_line.discount or 0.0) / 100.0)
            else:
                price = 0
            order = order_line.order_id
            return order_line.tax_id._origin.compute_all(price, order.currency_id, order_line.product_uom_qty,
                                                         product=order_line.product_id,
                                                         partner=order.partner_shipping_id)

        account_move = self.env['account.move']
        for order in self:
            tax_lines_data = account_move._prepare_tax_lines_data_for_totals_from_object(order.order_line,
                                                                                         compute_taxes)
            tax_totals = account_move._get_tax_totals(order.partner_id, tax_lines_data, order.amount_total,
                                                      order.amount_untaxed, order.currency_id)
            order.tax_totals_json = json.dumps(tax_totals)

    def action_confirm(self):
        def send_bom_create_notification(bom):
            notification_ids = [((0, 0, {
                'res_partner_id': self.env.user.partner_id.id,
                'notification_type': 'inbox'}))]
            user_id = self.env.user.id
            url = f"/web#id={bom.id}&view_type=form&model=mrp.bom"
            message = f"已建立 BOM <a href='{self.env['ir.config_parameter'].sudo().get_param('web.base.url')}{url}'>{bom.product_tmpl_id.name}</a> "
            group = self.env['mail.channel'].search([('group_public_id', '=', self.env.ref('kci-mrp.kci_mrp_group').id)])
            group.message_post(author_id=user_id,
                               body=message + "請查收",
                               message_type='notification',
                               subtype_xmlid="mail.mt_comment",
                               notification_ids=notification_ids,
                               partner_ids=[self.env.user.id],
                               notify_by_email=False,
                            )
            self.env['res.partner'].browse(user_id).message_post(author_id=user_id,
                                                                 body=message + "已寄送",
                                                                 message_type='notification',
                                                                 subtype_xmlid="mail.mt_comment",
                                                                 notification_ids=notification_ids,
                                                                 partner_ids=[self.env.user.id],
                                                                 notify_by_email=False,
                                                                 )
            email_to = ''
            for partner in group.channel_partner_ids:
                email_to += f'{partner.email}, '
            email_values = {
                'email_to': email_to,
                'email_cc': self.env.user.partner_id.email,
                'email_from': "erp_system@kci-global.com.tw",
                'body_html': message,
            }
            mail_template = self.env.ref('sales.bom2_email_template')
            mail_template.send_mail(self.id, force_send=True, email_values=email_values)

        def create_bom2_from_order():
            bom = self.env['mrp.bom'].create({
                'product_tmpl_id': self.inquiry_id.bom_data.product_tmpl_id.id,
                'status': 'external',
                'customer_id': self.inquiry_id.customer.id,
                'product_qty': self.inquiry_id.bom_data.product_qty,
                'inquiry_id': self.inquiry_id.id,
            })
            #for line in self.order_line:
            #    self.env['mrp.bom.line'].create({
            #        'product_id': line.product_id.id,
            #        'product_qty': line.product_uom_qty,
            #        'bom_id': bom.id,
            #        'material_source': 'stock',
            #    })
            return bom

        if self._get_forbidden_state_confirm() & set(self.mapped('state')):
            raise UserError(_(
                'It is not allowed to confirm an order in the following states: %s'
            ) % (', '.join(self._get_forbidden_state_confirm())))

        for order in self.filtered(lambda order: order.partner_id not in order.message_partner_ids):
            order.message_subscribe([order.partner_id.id])

        self.env['sale.order'].search([("id", "=", self.id)]).copy({'name': self.name})

        self.write(self._prepare_confirmation_values())

        # Context key 'default_name' is sometimes propagated up to here.
        # We don't need it and it creates issues in the creation of linked records.
        context = self._context.copy()
        context.pop('default_name', None)

        self.with_context(context)._action_confirm()
        if self.env.user.has_group('sale.group_auto_done_setting'):
            self.action_done()

        bom = create_bom2_from_order()
        send_bom_create_notification(bom)

        return True

    def action_copy(self):
        self.env['sale.order'].search([("id", "=", self.id)]).copy({'name': self.name, 'version': str(int(self.version) + 1)})
        return True

    def action_split(self):
        current_order = self.env['sale.order'].search([("id", "=", self.id)])
        v1 = self.env['sale.order'].search([("id", "=", self.id)]).copy({'name': f'{self.name}-001', 'order_code': f'{self.order_code}-001', 'version': str(int(self.version) + 1)})
        v2 = self.env['sale.order'].search([("id", "=", self.id)]).copy({'name': f'{self.name}-002', 'order_code': f'{self.order_code}-002', 'version': str(int(self.version) + 1)})
        v1.parent_order = current_order
        v2.parent_order = current_order
        return True


class BankInfo(models.Model):
    _name = 'sales.order.bank'
    _description = 'bank information for quotation and order'

    account_name = fields.Char(string='戶名', size=100)
    name = fields.Char(string='銀行', size=100)
    account_sub_bank = fields.Char(string='分行', size=100)
    account = fields.Char(string='帳號', size=100)


class OrderAttached(models.Model):
    _name = 'sales.order.attached'
    _description = 'attached for sales order'

    attached_file = fields.Binary('Attached', tracking=True)
    attached_file_name = fields.Char('Attached Name')

    order_id = fields.Many2one('sale.order', 'attached_id', index=True, ondelete='cascade')

    company_id = fields.Many2one(
        'res.company', 'Company', index=True,
        default=lambda self: self.env.company)
